package com.intershop.support.component.mycoupon.internal;

import java.nio.charset.Charset;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;

import com.google.inject.Inject;
import com.intershop.beehive.core.capi.util.ObjectMapper;
import com.intershop.beehive.orm.capi.common.ORMObjectCollection;
import com.intershop.component.repository.capi.AbstractDomainRepositoryBOExtension;
import com.intershop.component.repository.capi.RepositoryBO;
import com.intershop.support.component.mycoupon.capi.MyCouponBO;
import com.intershop.support.component.mycoupon.capi.MyCouponBORepository;

public class MyCouponBORepositoryImpl extends AbstractDomainRepositoryBOExtension
                implements MyCouponBORepository, ObjectMapper<MyCouponPO, MyCouponBO>
{
    /**
     * Stores all MyCoupons to it's persistance objects
     */
    private Map<MyCouponPO, MyCouponBO> cache = new HashMap<>();

    @Inject
    private MyCouponPOFactory myCouponPOFactory;
    
    public MyCouponBORepositoryImpl(String extensionID, RepositoryBO extendedObject)
    {
        super(extensionID, extendedObject);
    }

    /**
     * Returns the {@link MyCouponBO} for it's persistance object. If it's not existing, a new my coupon is created.
     * 
     * @param myCouponPO
     *                            - persistance object, that should converted to its business object
     * @return the {@link MyCouponBO} for the given persistance object
     */
    @Override
    public synchronized MyCouponBO resolve(MyCouponPO myCouponPO)
    {
        if (this.cache.containsKey(myCouponPO))
        {
            return this.cache.get(myCouponPO);
        }

        MyCouponBO myCouponBO = new MyCouponBOImpl(myCouponPO, getContext());
        cache.put(myCouponPO, myCouponBO);

        return myCouponBO;
    }

    /**
     * for the given parameters a new MyCoupon will be created
     */
    @Override
    public MyCouponBO createMyCouponBOByEmail(String email)
    {
        // you can validate the code and so on
        MyCouponPO myCouponPO = this.myCouponPOFactory.create(getDomain(), email, generateCode(), new java.util.Date());
        MyCouponBO myCouponBO = this.resolve(myCouponPO);
        cache.put(myCouponPO, myCouponBO);
        return myCouponBO;
    }      

    private String generateCode() 
    {
        byte[] array = new byte[7];
        new Random().nextBytes(array);
        return new String(array, Charset.forName("UTF-8")); 
    }
    
    @Override
    public int countMyCouponsByEmail(String email)
    {
        @SuppressWarnings("unchecked")
        final Collection<MyCouponPO> myCouponPOs = this.myCouponPOFactory
                        .getObjectsBySQLWhere("email = ? ", new String[] { email });

        return myCouponPOs.size();
    }

    /**
     * Generates a coupon code
     * @return the generated coupon code
     */
    @Override
    public MyCouponBO getMyCouponBOByCode(String couponCode)
    {
        @SuppressWarnings("unchecked")
        final ORMObjectCollection<MyCouponPO> myCouponPOs = this.myCouponPOFactory
                        .getObjectsBySQLWhere("coupon=?", new Object[] { couponCode });

        if (myCouponPOs.isEmpty())
        {
            return null;
        }
        return resolve(myCouponPOs.iterator().next());
    }
}
